import ts = require('typescript');
import { OTree, UnknownSyntax } from './o-tree';
import { ImportStatement } from './typescript/imports';
/**
 * AST conversion operation
 *
 * Dispatch the actual conversion to a specific handler which will get the
 * appropriate method called for particular AST nodes. The handler may use
 * context to modify its own operations when traversing the tree hierarchy,
 * the type of which should be expressed via the C parameter.
 */
export declare class AstConverter<C> {
    private readonly sourceFile;
    private readonly typeChecker;
    private readonly handler;
    private readonly options;
    readonly diagnostics: ts.Diagnostic[];
    readonly currentContext: C;
    constructor(sourceFile: ts.SourceFile, typeChecker: ts.TypeChecker, handler: AstHandler<C>, options?: ConvertOptions);
    /**
     * Merge the new context with the current context and create a new Converter from it
     */
    updateContext(contextUpdate: C): AstConverter<C>;
    /**
     * Convert a single node to an OTree
     */
    convert(node: ts.Node | undefined): OTree;
    /**
     * Convert a set of nodes, filtering out hidden nodes
     */
    convertAll(nodes: ReadonlyArray<ts.Node>): OTree[];
    /**
     * Convert a set of nodes, but update the context for the last one.
     *
     * Takes visibility into account.
     */
    convertLastDifferently(nodes: ReadonlyArray<ts.Node>, lastContext: C): OTree[];
    textOf(node: ts.Node): string;
    textAt(pos: number, end: number): string;
    typeOfExpression(node: ts.Expression): ts.Type | undefined;
    typeOfType(node: ts.TypeNode): ts.Type;
    report(node: ts.Node, messageText: string, category?: ts.DiagnosticCategory): void;
    reportUnsupported(node: ts.Node): void;
    /**
     * Return a newline if the given node is preceded by at least one newline
     *
     * Used to mirror newline use between matchin brackets (such as { ... } and [ ... ]).
     */
    mirrorNewlineBefore(viz?: ts.Node, suffix?: string): string;
    /**
     * Dispatch node to handler
     */
    private dispatch;
    /**
     * Attach any leading whitespace and comments to the given output tree
     *
     * Regardless of whether it's declared to be able to accept such or not.
     */
    private attachLeadingTrivia;
}
/**
 * Interface for AST handlers
 *
 * C is the type of hierarchical context the handler uses. Context
 * needs 2 operations: a constructor for a default context, and a
 * merge operation to combine 2 contexts to yield a new one.
 *
 * Otherwise, the handler should return an OTree for every type
 * of AST node.
 */
export interface AstHandler<C> {
    readonly defaultContext: C;
    mergeContext(old: C, update: C): C;
    sourceFile(node: ts.SourceFile, context: AstConverter<C>): OTree;
    commentRange(node: ts.CommentRange, context: AstConverter<C>): OTree;
    importStatement(node: ImportStatement, context: AstConverter<C>): OTree;
    stringLiteral(node: ts.StringLiteral, children: AstConverter<C>): OTree;
    functionDeclaration(node: ts.FunctionDeclaration, children: AstConverter<C>): OTree;
    identifier(node: ts.Identifier, children: AstConverter<C>): OTree;
    block(node: ts.Block, children: AstConverter<C>): OTree;
    parameterDeclaration(node: ts.ParameterDeclaration, children: AstConverter<C>): OTree;
    returnStatement(node: ts.ReturnStatement, context: AstConverter<C>): OTree;
    binaryExpression(node: ts.BinaryExpression, context: AstConverter<C>): OTree;
    ifStatement(node: ts.IfStatement, context: AstConverter<C>): OTree;
    propertyAccessExpression(node: ts.PropertyAccessExpression, context: AstConverter<C>): OTree;
    callExpression(node: ts.CallExpression, context: AstConverter<C>): OTree;
    expressionStatement(node: ts.ExpressionStatement, context: AstConverter<C>): OTree;
    token<A extends ts.SyntaxKind>(node: ts.Token<A>, context: AstConverter<C>): OTree;
    objectLiteralExpression(node: ts.ObjectLiteralExpression, context: AstConverter<C>): OTree;
    newExpression(node: ts.NewExpression, context: AstConverter<C>): OTree;
    propertyAssignment(node: ts.PropertyAssignment, context: AstConverter<C>): OTree;
    variableStatement(node: ts.VariableStatement, context: AstConverter<C>): OTree;
    variableDeclarationList(node: ts.VariableDeclarationList, context: AstConverter<C>): OTree;
    variableDeclaration(node: ts.VariableDeclaration, context: AstConverter<C>): OTree;
    jsDoc(node: ts.JSDoc, context: AstConverter<C>): OTree;
    arrayLiteralExpression(node: ts.ArrayLiteralExpression, context: AstConverter<C>): OTree;
    shorthandPropertyAssignment(node: ts.ShorthandPropertyAssignment, context: AstConverter<C>): OTree;
    forOfStatement(node: ts.ForOfStatement, context: AstConverter<C>): OTree;
    classDeclaration(node: ts.ClassDeclaration, context: AstConverter<C>): OTree;
    constructorDeclaration(node: ts.ConstructorDeclaration, context: AstConverter<C>): OTree;
    propertyDeclaration(node: ts.PropertyDeclaration, context: AstConverter<C>): OTree;
    methodDeclaration(node: ts.MethodDeclaration, context: AstConverter<C>): OTree;
    interfaceDeclaration(node: ts.InterfaceDeclaration, context: AstConverter<C>): OTree;
    propertySignature(node: ts.PropertySignature, context: AstConverter<C>): OTree;
    asExpression(node: ts.AsExpression, context: AstConverter<C>): OTree;
    prefixUnaryExpression(node: ts.PrefixUnaryExpression, context: AstConverter<C>): OTree;
    spreadElement(node: ts.SpreadElement, context: AstConverter<C>): OTree;
    spreadAssignment(node: ts.SpreadAssignment, context: AstConverter<C>): OTree;
    templateExpression(node: ts.TemplateExpression, context: AstConverter<C>): OTree;
    nonNullExpression(node: ts.NonNullExpression, context: AstConverter<C>): OTree;
    parenthesizedExpression(node: ts.ParenthesizedExpression, context: AstConverter<C>): OTree;
    maskingVoidExpression(node: ts.VoidExpression, context: AstConverter<C>): OTree;
    noSubstitutionTemplateLiteral(node: ts.NoSubstitutionTemplateLiteral, context: AstConverter<C>): OTree;
    ellipsis(node: ts.SpreadElement | ts.SpreadAssignment, context: AstConverter<C>): OTree;
}
export declare function nimpl<C>(node: ts.Node, context: AstConverter<C>, options?: {
    additionalInfo?: string;
}): UnknownSyntax;
export interface ConvertOptions {
    /**
     * If enabled, don't translate the text of unknown nodes
     *
     * @default true
     */
    bestEffort?: boolean;
}
